@@ -45,12 +45,20 @@
 
 typedef struct
 {
-    GtkWidget *log_level;
-    GtkWidget *log_level_apps;
-    GtkWidget *log_level_apps_label;
-    GtkWidget *infobar_label;
+    GtkWidget  *log_level;
+    GtkWidget  *log_level_apps;
+    GtkWidget  *log_level_apps_label;
+    GtkWidget  *infobar_label;
+    GtkWidget  *log_listbox;
+    GtkToolbar *log_toolbar;
 } NotificationLogWidgets;
 
+typedef struct
+{
+    GtkWidget  *do_slideout;
+    GtkWidget  *do_slideout_label;
+} NotificationSlideoutWidgets;
+
 static void
 xfce_notifyd_config_show_notification_callback(NotifyNotification *notification,
                                                const char         *action,
@@ -248,10 +256,11 @@ xfce_notifyd_config_dialog_response(GtkWidget *dialog, gint response, gpointer u
 }
 
 static void
-xfce_notifyd_config_preview_clicked(GtkButton *button,
-                                    GtkWidget *dialog)
+xfce_notifyd_config_preview_clicked(GtkButton *button)
 {
-    xfce_notifyd_config_show_notification_preview(GTK_WINDOW(dialog));
+  GtkWidget *window = gtk_widget_get_toplevel (GTK_WIDGET (button));
+
+  xfce_notifyd_config_show_notification_preview (GTK_WINDOW (window));
 }
 
 /* Shows a separator before each row. */
@@ -305,10 +314,11 @@ xfce4_notifyd_mute_application (GtkListBox *known_applications_listbox,
             GValue *muted_application;
             muted_application = g_ptr_array_index (muted_applications, i);
             if (g_str_match_string (g_value_get_string (val), g_value_get_string (muted_application), FALSE) == TRUE) {
-                g_ptr_array_remove_index (muted_applications, i);
+                if (!g_ptr_array_remove_index (muted_applications, i))
+                    g_warning ("Could not remove %s from the list of muted applications. %i", new_app_name, i);
                 break;
             }
-            g_warning ("Could not remove %s from the list of muted applications.", new_app_name);
+
         }
 
     }
@@ -391,18 +401,39 @@ xfce4_notifyd_known_applications_changed (XfconfChannel *channel,
 
     if (known_applications != NULL) {
         for (i = 0; i < known_applications->len; i++) {
+            GdkPixbuf *pix = NULL;
+            GtkIconInfo *icon_info = NULL;
+            GtkIconInfo *icon_info_lower = NULL;
+            gchar const *desktop_icon_name = NULL;
+            gchar *icon_name_lower;
+
             known_application = g_ptr_array_index (known_applications, i);
             hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
             label = gtk_label_new (g_value_get_string (known_application));
-            icon_name = g_value_get_string (known_application);
-            if (gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default(), icon_name, 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK))
-                icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
-            else if (gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default(), g_ascii_strdown(icon_name, -1), 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK))
-                icon = gtk_image_new_from_icon_name (g_ascii_strdown(icon_name, -1), GTK_ICON_SIZE_MENU);
-            else {
-                icon = gtk_image_new ();
-                gtk_image_set_pixel_size (GTK_IMAGE(icon), 16);
+            /* Make sure spaces are converted to dashes so GTK_ICON_LOOKUP_GENERIC_FALLBACK works as expected */
+            icon_name = g_strdelimit ((gchar *) g_value_get_string (known_application)," ",'-');
+            icon_name_lower = g_ascii_strdown (icon_name, -1);
+            icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default(), icon_name, 24, GTK_ICON_LOOKUP_GENERIC_FALLBACK);
+            icon_info_lower = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default(), g_ascii_strdown (icon_name, -1), 24, GTK_ICON_LOOKUP_GENERIC_FALLBACK);
+            desktop_icon_name = notify_icon_name_from_desktop_id (icon_name_lower);
+            /* Find icons in the right priority: normal icon name with fallback, lowercase icon name with fallback,
+               Desktop file icon property or empty. */
+            if (icon_info) {
+                pix = gtk_icon_info_load_icon (icon_info, NULL);
+                icon = gtk_image_new_from_pixbuf (gdk_pixbuf_scale_simple (pix, 24, 24, GDK_INTERP_BILINEAR));
             }
+            else if (icon_info_lower) {
+                pix = gtk_icon_info_load_icon (icon_info_lower, NULL);
+                icon = gtk_image_new_from_pixbuf (gdk_pixbuf_scale_simple (pix, 24, 24, GDK_INTERP_BILINEAR));
+            }
+            else if (desktop_icon_name)
+                icon = gtk_image_new_from_icon_name (desktop_icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
+            else
+                icon = gtk_image_new ();
+            g_free (icon_name_lower);
+            if (pix)
+                g_object_unref (G_OBJECT (pix));
+            gtk_image_set_pixel_size (GTK_IMAGE (icon), 24);
 
 #if GTK_CHECK_VERSION (3, 16, 0)
             gtk_label_set_xalign (GTK_LABEL (label), 0);
@@ -411,6 +442,7 @@ xfce4_notifyd_known_applications_changed (XfconfChannel *channel,
 #endif
             mute_switch = gtk_switch_new ();
             gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
+            gtk_widget_set_valign (mute_switch, GTK_ALIGN_CENTER);
             gtk_switch_set_active (GTK_SWITCH (mute_switch), TRUE);
             gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, TRUE, 3);
             gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 3);
@@ -449,6 +481,20 @@ xfce4_notifyd_do_not_disturb_activated (GtkSwitch *do_not_disturb_switch,
     gtk_switch_set_active (GTK_SWITCH (do_not_disturb_switch), state);
 }
 
+static void
+xfce4_notifyd_do_fadeout_activated (GtkSwitch *do_fadeout,
+                                    gboolean state,
+                                    gpointer user_data)
+{
+    NotificationSlideoutWidgets *slideout_widgets = user_data;
+
+    /* The sliding out animation is only available along with fade-out */
+    if (state == FALSE)
+        gtk_switch_set_active (GTK_SWITCH (slideout_widgets->do_slideout), FALSE);
+    gtk_widget_set_sensitive (slideout_widgets->do_slideout, state);
+    gtk_widget_set_sensitive (slideout_widgets->do_slideout_label, state);
+    gtk_switch_set_active (GTK_SWITCH (do_fadeout), state);
+}
 
 static void
 xfce4_notifyd_log_activated (GtkSwitch *log_switch,
@@ -470,16 +516,22 @@ xfce4_notifyd_log_activated (GtkSwitch *log_switch,
 
 static void
 xfce4_notifyd_log_open (GtkButton *button, gpointer user_data) {
-    gchar *notify_log_path = NULL;
+    gchar *notify_log_path;
 
     notify_log_path = xfce_resource_lookup (XFCE_RESOURCE_CACHE, XFCE_NOTIFY_LOG_FILE);
-    if (!g_app_info_launch_default_for_uri (g_strdup_printf("file://%s", notify_log_path), NULL, NULL))
-        g_warning ("Could not open the log file: %s", notify_log_path);
+    if (notify_log_path) {
+        gchar *uri = g_strdup_printf ("file://%s", notify_log_path);
+        if (!g_app_info_launch_default_for_uri (uri, NULL, NULL))
+            g_warning ("Could not open the log file: %s", notify_log_path);
+        g_free (uri);
+        g_free (notify_log_path);
+    }
 }
 
 static void
-xfce4_notifyd_log_populate (GtkWidget *log_listbox)
+xfce4_notifyd_log_populate (NotificationLogWidgets *log_widgets)
 {
+    GtkWidget *log_listbox = log_widgets->log_listbox;
     GKeyFile *notify_log;
     GtkCallback func = listbox_remove_all;
     gint i;
@@ -487,17 +539,22 @@ xfce4_notifyd_log_populate (GtkWidget *log_listbox)
     gchar *timestamp;
     gchar *limit_label;
     GtkWidget *limit_button;
+    gsize num_groups = 0;
+    GdkPixbuf *pixbuf = NULL;
+    gchar *notify_log_icon_folder;
+    gchar *notify_log_icon_path;
 
     today = g_date_time_new_now_local ();
     timestamp = g_date_time_format (today, "%F");
 
     gtk_container_foreach (GTK_CONTAINER (log_listbox), func, log_listbox);
     notify_log = xfce_notify_log_get();
+    notify_log_icon_folder = xfce_resource_save_location (XFCE_RESOURCE_CACHE,
+                                                          XFCE_NOTIFY_ICON_PATH, TRUE);
 
     if (notify_log) {
         gchar **groups;
         gboolean yesterday = FALSE;
-        gsize num_groups;
         int log_length;
 
         groups = g_key_file_get_groups (notify_log, &num_groups);
@@ -516,6 +573,7 @@ xfce4_notifyd_log_populate (GtkWidget *log_listbox)
             char *markup;
             gchar *app_name;
             gchar *tooltip_timestamp;
+            gchar *tmp;
             GTimeVal tv;
             GDateTime *log_timestamp;
 
@@ -537,35 +595,71 @@ xfce4_notifyd_log_populate (GtkWidget *log_listbox)
             }
 
             app_name = g_key_file_get_string (notify_log, group, "app_name", NULL);
-            markup = g_markup_printf_escaped (format, g_key_file_get_string (notify_log, group, "summary", NULL));
+            tmp = g_key_file_get_string (notify_log, group, "summary", NULL);
+            markup = g_markup_printf_escaped (format, tmp);
+            g_free (tmp);
             summary = gtk_label_new (NULL);
             gtk_label_set_markup (GTK_LABEL (summary), markup);
+#if GTK_CHECK_VERSION (3, 16, 0)
             gtk_label_set_xalign (GTK_LABEL (summary), 0);
+#else
+            gtk_widget_set_halign (summary, GTK_ALIGN_START);
+#endif
             g_free (markup);
-            body = gtk_label_new (g_key_file_get_string (notify_log, group, "body", NULL));
+            tmp = g_key_file_get_string (notify_log, group, "body", NULL);
+            body = gtk_label_new (NULL);
+            if (pango_parse_markup (tmp, -1, 0, NULL, NULL, NULL, NULL)) {
+                gtk_label_set_markup (GTK_LABEL (body), tmp);
+            } else {
+                gchar *tmp1;
+
+                tmp1 = g_markup_escape_text (tmp, -1);
+                gtk_label_set_text (GTK_LABEL (body), tmp1);
+                g_free (tmp1);
+            }
+            g_free (tmp);
+#if GTK_CHECK_VERSION (3, 16, 0)
             gtk_label_set_xalign (GTK_LABEL (body), 0);
+#else
+            gtk_widget_set_halign (body, GTK_ALIGN_START);
+#endif
             gtk_label_set_ellipsize (GTK_LABEL (body), PANGO_ELLIPSIZE_END);
-            app_icon = gtk_image_new_from_icon_name (g_key_file_get_string (notify_log, group, "app_icon", NULL),
-                                                     GTK_ICON_SIZE_LARGE_TOOLBAR);
-            gtk_image_set_pixel_size (GTK_IMAGE (app_icon), 24);
+            tmp = g_key_file_get_string (notify_log, group, "app_icon", NULL);
+            notify_log_icon_path = g_strconcat (notify_log_icon_folder , tmp, ".png", NULL);
+            if (g_file_test (notify_log_icon_path, G_FILE_TEST_EXISTS)) {
+                pixbuf = gdk_pixbuf_new_from_file_at_scale (notify_log_icon_path,
+                                                            24, 24, FALSE, NULL);
+                app_icon = gtk_image_new_from_pixbuf (pixbuf);
+                if (pixbuf)
+                    g_object_unref (pixbuf);
+            } else {
+                app_icon = gtk_image_new_from_icon_name (tmp, GTK_ICON_SIZE_LARGE_TOOLBAR);
+                gtk_image_set_pixel_size (GTK_IMAGE (app_icon), 24);
+            }
+            g_free (notify_log_icon_path);
+            g_free (tmp);
             gtk_widget_set_margin_start (app_icon, 3);
-            expire_timeout = gtk_label_new (g_key_file_get_string (notify_log, group, "expire-timeout", NULL));
+            tmp = g_key_file_get_string (notify_log, group, "expire-timeout", NULL);
+            expire_timeout = gtk_label_new (tmp);
+            g_free (tmp);
             // TODO: actions and timeout are missing (timeout is only interesting for urgent messages) - do we need that?
             grid = gtk_grid_new ();
             gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
             gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (app_icon), 0, 0 , 1, 2);
 
             /* Handle icon-only notifications */
-            if (g_strcmp0 (g_key_file_get_string (notify_log, group, "body", NULL), "") == 0) {
+            tmp = g_key_file_get_string (notify_log, group, "body", NULL);
+            if (g_strcmp0 (tmp, "") == 0) {
                 gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (summary), 1, 0, 1, 2);
                 markup = g_strdup_printf (tooltip_format_simple, app_name, tooltip_timestamp);
             }
             else {
                 gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (summary), 1, 0, 1, 1);
                 gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (body), 1, 1, 1, 1);
-                markup = g_strdup_printf (tooltip_format, app_name, tooltip_timestamp,
-                                          g_key_file_get_string (notify_log, group, "body", NULL));
+                markup = g_strdup_printf (tooltip_format, app_name, tooltip_timestamp, tmp);
             }
+            g_free (tmp);
+            g_free (app_name);
 
             gtk_widget_set_tooltip_markup (grid, markup);
             g_free (markup);
@@ -583,25 +677,32 @@ xfce4_notifyd_log_populate (GtkWidget *log_listbox)
                               G_CALLBACK (xfce4_notifyd_log_open), log_listbox);
             gtk_list_box_insert (GTK_LIST_BOX (log_listbox), limit_button, LOG_DISPLAY_LIMIT + 1);
         }
+        g_strfreev (groups);
         g_key_file_free (notify_log);
     }
 
+    g_free(notify_log_icon_folder);
+
     gtk_widget_show_all (log_listbox);
+
+    /* Update "open file" and "clear log" buttons sensitivity */
+    gtk_widget_set_sensitive (GTK_WIDGET (gtk_toolbar_get_nth_item (log_widgets->log_toolbar, 1)), notify_log != NULL);
+    gtk_widget_set_sensitive (GTK_WIDGET (gtk_toolbar_get_nth_item (log_widgets->log_toolbar, 2)), num_groups > 0);
 }
 
 static void
 xfce4_notifyd_log_refresh (GtkButton *button, gpointer user_data) {
-    GtkWidget *log_listbox = user_data;
-    xfce4_notifyd_log_populate (log_listbox);
+    xfce4_notifyd_log_populate (user_data);
 }
 
 static void xfce_notify_log_clear_button_clicked (GtkButton *button, gpointer user_data) {
-    GtkWidget *log_listbox = user_data;
+    GtkWidget *log_listbox = ((NotificationLogWidgets *) user_data)->log_listbox;
     GtkCallback func = listbox_remove_all;
 
     /* Clear the log file and empty the listbox widget */
     gtk_container_foreach (GTK_CONTAINER (log_listbox), func, log_listbox);
-    xfce_notify_log_clear();
+    xfce_notify_log_clear ();
+    xfce4_notifyd_log_populate (user_data);
 }
 
 static void xfce4_notifyd_show_help(GtkButton *button,
@@ -610,6 +711,25 @@ static void xfce4_notifyd_show_help(GtkButton *button,
     xfce_dialog_show_help_with_version(GTK_WINDOW(dialog), "notifyd", "start", NULL, NULL);
 }
 
+static void xfce_notify_bus_name_appeared_cb (GDBusConnection *connection,
+                                              const gchar *name,
+                                              const gchar *name_owner,
+                                              gpointer user_data)
+{
+    GtkWidget  *notifyd_running = user_data;
+
+    gtk_revealer_set_reveal_child (GTK_REVEALER (notifyd_running), FALSE);
+}
+
+static void xfce_notify_bus_name_vanished_cb (GDBusConnection *connection,
+                                              const gchar *name,
+                                              gpointer user_data)
+{
+    GtkWidget *notifyd_running = user_data;
+
+    gtk_revealer_set_reveal_child (GTK_REVEALER (notifyd_running), TRUE);
+}
+
 GtkWidget *
 placeholder_label_new (gchar *place_holder_text)
 {
@@ -643,8 +763,6 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
     GtkWidget *do_not_disturb_info;
     GtkWidget *log_switch;
     GtkWidget *log_scrolled_window;
-    GtkWidget *log_listbox;
-    GtkToolbar *log_toolbar;
     GtkToolItem *log_clear_button;
     GtkToolItem *log_refresh_button;
     GtkToolItem *log_open_button;
@@ -656,6 +774,7 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
     gchar *current_theme;
     GKeyFile *notification_log;
     NotificationLogWidgets log_widgets;
+    NotificationSlideoutWidgets slideout_widgets;
 
     gtk_builder_connect_signals(builder, NULL);
 
@@ -732,11 +851,22 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
 
     do_fadeout = GTK_WIDGET(gtk_builder_get_object(builder, "do_fadeout"));
     xfconf_g_property_bind(channel, "/do-fadeout", G_TYPE_BOOLEAN,
-                          G_OBJECT(do_fadeout), "active");
+                           G_OBJECT(do_fadeout), "active");
+
+    slideout_widgets.do_slideout_label = GTK_WIDGET(gtk_builder_get_object(builder, "do_slideout_label"));
+    slideout_widgets.do_slideout = GTK_WIDGET(gtk_builder_get_object(builder, "do_slideout"));
+    xfconf_g_property_bind(channel, "/do-slideout", G_TYPE_BOOLEAN,
+                           G_OBJECT(slideout_widgets.do_slideout), "active");
+    g_signal_connect (G_OBJECT (do_fadeout), "state-set",
+                      G_CALLBACK (xfce4_notifyd_do_fadeout_activated), &slideout_widgets);
+    if (gtk_switch_get_active (GTK_SWITCH (do_fadeout)) == FALSE) {
+        gtk_widget_set_sensitive (slideout_widgets.do_slideout_label, FALSE);
+        gtk_widget_set_sensitive (slideout_widgets.do_slideout, FALSE);
+    }
 
     btn = GTK_WIDGET(gtk_builder_get_object(builder, "preview_button"));
     g_signal_connect(G_OBJECT(btn), "clicked",
-                     G_CALLBACK(xfce_notifyd_config_preview_clicked), dlg);
+                     G_CALLBACK(xfce_notifyd_config_preview_clicked), NULL);
 
     /*******************
         APPLICATIONS   *
@@ -785,35 +915,36 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
     xfce4_notifyd_log_activated (GTK_SWITCH (log_switch), gtk_switch_get_active (GTK_SWITCH(log_switch)), &log_widgets);
 
     log_scrolled_window = GTK_WIDGET (gtk_builder_get_object (builder, "log_scrolled_window"));
-    log_listbox = gtk_list_box_new ();
-    gtk_container_add (GTK_CONTAINER (log_scrolled_window), log_listbox);
-    gtk_list_box_set_header_func (GTK_LIST_BOX (log_listbox), display_header_func, NULL, NULL);
-    placeholder_label = placeholder_label_new ("<big><b>Empty log</b></big>"
-                                               "\nNo notifications have been logged yet.");
-    gtk_list_box_set_placeholder (GTK_LIST_BOX (log_listbox), placeholder_label);
+    log_widgets.log_listbox = gtk_list_box_new ();
+    gtk_container_add (GTK_CONTAINER (log_scrolled_window), log_widgets.log_listbox);
+    gtk_list_box_set_header_func (GTK_LIST_BOX (log_widgets.log_listbox), display_header_func, NULL, NULL);
+    placeholder_label = placeholder_label_new (_("<big><b>Empty log</b></big>"
+                                                 "\nNo notifications have been logged yet."));
+    gtk_list_box_set_placeholder (GTK_LIST_BOX (log_widgets.log_listbox), placeholder_label);
     gtk_widget_show_all (placeholder_label);
-    xfce4_notifyd_log_populate (log_listbox);
 
-    log_toolbar = GTK_TOOLBAR (gtk_builder_get_object (builder, "log_toolbar"));
+    log_widgets.log_toolbar = GTK_TOOLBAR (gtk_builder_get_object (builder, "log_toolbar"));
     icon = gtk_image_new_from_icon_name ("view-refresh-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
     log_refresh_button = gtk_tool_button_new (icon, _("Refresh"));
     gtk_widget_set_tooltip_text (GTK_WIDGET (log_refresh_button), _("Refresh the notification log"));
-    gtk_toolbar_insert(log_toolbar, GTK_TOOL_ITEM(log_refresh_button), 0);
+    gtk_toolbar_insert(log_widgets.log_toolbar, GTK_TOOL_ITEM(log_refresh_button), 0);
     g_signal_connect (G_OBJECT (log_refresh_button), "clicked",
-                      G_CALLBACK (xfce4_notifyd_log_refresh), log_listbox);
+                      G_CALLBACK (xfce4_notifyd_log_refresh), &log_widgets);
     icon = gtk_image_new_from_icon_name ("document-open-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
     log_open_button = gtk_tool_button_new (icon, _("Open"));
     gtk_widget_set_tooltip_text (GTK_WIDGET (log_open_button), _("Open the notification log in an external editor"));
-    gtk_toolbar_insert(log_toolbar, GTK_TOOL_ITEM(log_open_button), 1);
+    gtk_toolbar_insert(log_widgets.log_toolbar, GTK_TOOL_ITEM(log_open_button), 1);
     g_signal_connect (G_OBJECT (log_open_button), "clicked",
-                      G_CALLBACK (xfce4_notifyd_log_open), log_listbox);
+                      G_CALLBACK (xfce4_notifyd_log_open), log_widgets.log_listbox);
     icon = gtk_image_new_from_icon_name ("edit-clear-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
     log_clear_button = gtk_tool_button_new (icon, _("Clear"));
     gtk_widget_set_tooltip_text (GTK_WIDGET (log_clear_button), _("Clear the notification log"));
-    gtk_toolbar_insert(log_toolbar, GTK_TOOL_ITEM(log_clear_button), 2);
+    gtk_toolbar_insert(log_widgets.log_toolbar, GTK_TOOL_ITEM(log_clear_button), 2);
     g_signal_connect (G_OBJECT (log_clear_button), "clicked",
-                      G_CALLBACK (xfce_notify_log_clear_button_clicked), log_listbox);
-    gtk_widget_show_all (GTK_WIDGET(log_toolbar));
+                      G_CALLBACK (xfce_notify_log_clear_button_clicked), &log_widgets);
+    gtk_widget_show_all (GTK_WIDGET(log_widgets.log_toolbar));
+
+    xfce4_notifyd_log_populate (&log_widgets);
 
     return dlg;
 }
@@ -823,9 +954,11 @@ main(int argc,
      char **argv)
 {
     GtkWidget *settings_dialog = NULL;
+    GtkWidget *notifyd_running;
     GtkBuilder *builder;
     gboolean opt_version = FALSE;
     gint32 opt_socket_id = 0;
+    guint watch_handle_id;
     GOptionEntry option_entries[] = {
         { "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version, N_("Display version information"), NULL },
         { "socket-id", 's', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_INT, &opt_socket_id, N_("Settings manager socket"), N_("SOCKET_ID") },
@@ -873,6 +1006,16 @@ main(int argc,
 
     settings_dialog = xfce4_notifyd_config_setup_dialog(builder);
 
+    notifyd_running = GTK_WIDGET (gtk_builder_get_object (builder, "notifyd_running"));
+    gtk_revealer_set_reveal_child (GTK_REVEALER (notifyd_running), FALSE);
+    watch_handle_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+                                        "org.freedesktop.Notifications",
+                                        G_BUS_NAME_WATCHER_FLAGS_NONE,
+                                        xfce_notify_bus_name_appeared_cb,
+                                        xfce_notify_bus_name_vanished_cb,
+                                        notifyd_running,
+                                        NULL);
+
     if(opt_socket_id) {
         GtkWidget *plug, *plug_child;
 
@@ -898,6 +1041,7 @@ main(int argc,
         gtk_dialog_run(GTK_DIALOG(settings_dialog));
     }
 
+    g_bus_unwatch_name (watch_handle_id);
     notify_uninit();
     xfconf_shutdown();
 
