@@ -43,6 +43,7 @@
 
 #include "xfce-notify-daemon.h"
 #include "xfce-notify-window.h"
+#include "xfce-notify-log.h"
 #include "xfce-notify-marshal.h"
 
 #define SPACE 16
@@ -60,8 +61,12 @@ struct _XfceNotifyDaemon
     gdouble initial_opacity;
     GtkCornerType notify_location;
     gboolean do_fadeout;
-    gboolean primary_monitor;
+    gboolean do_slideout;
     gboolean do_not_disturb;
+    gboolean notification_log;
+    gint primary_monitor;
+    gint log_level;
+    gint log_level_apps;
 
     GtkCssProvider *css_provider;
     gboolean is_default_theme;
@@ -69,12 +74,8 @@ struct _XfceNotifyDaemon
     XfconfChannel *settings;
 
     GTree *active_notifications;
-    GList ***reserved_rectangles;
-    GdkRectangle **monitors_workarea;
-
-    gint changed_screen;
-
-    guint close_timeout;
+    GList **reserved_rectangles;
+    GdkRectangle *monitors_workarea;
 
     guint32 last_notification_id;
 };
@@ -114,10 +115,6 @@ static void xfce_notify_daemon_get_workarea(GdkScreen *screen,
                                             guint monitor,
                                             GdkRectangle *rect);
 static void daemon_quit (XfceNotifyDaemon *xndaemon);
-static gboolean xfce_notify_daemon_close_timeout(gpointer data);
-
-
-static GdkPixbuf *notify_pixbuf_from_image_data(GVariant *image_data);
 
 /* DBus method callbacks  forward declarations */
 static gboolean notify_get_capabilities (XfceNotifyGBus *skeleton,
@@ -192,6 +189,62 @@ xfce_notify_daemon_get_n_monitors_quark(void)
     return quark;
 }
 
+#if GTK_CHECK_VERSION (3, 22, 0)
+static gint
+xfce_notify_daemon_get_monitor_index (GdkDisplay *display,
+                                      GdkMonitor *monitor)
+{
+    gint i, nmonitors;
+
+    nmonitors = gdk_display_get_n_monitors (display);
+
+    for (i = 0; i < nmonitors; i++) {
+        if (monitor == gdk_display_get_monitor (display, i))
+            return i;
+    }
+
+    return 0;
+}
+#endif
+
+static gint
+xfce_notify_daemon_get_primary_monitor (GdkScreen *screen)
+{
+#if GTK_CHECK_VERSION (3, 22, 0)
+    GdkDisplay *display = gdk_screen_get_display (screen);
+    GdkMonitor *monitor =gdk_display_get_primary_monitor (display);
+
+    return xfce_notify_daemon_get_monitor_index (display, monitor);
+#else
+    return gdk_screen_get_primary_monitor(widget_screen);
+#endif
+}
+
+static gint
+xfce_notify_daemon_get_monitor_at_point (GdkScreen *screen,
+                                         gint x,
+                                         gint y)
+{
+#if GTK_CHECK_VERSION (3, 22, 0)
+    GdkDisplay *display = gdk_screen_get_display (screen);
+    GdkMonitor *monitor = gdk_display_get_monitor_at_point (display, x, y);
+
+    return xfce_notify_daemon_get_monitor_index (display, monitor);
+#else
+    return gdk_screen_get_monitor_at_point(widget_screen);
+#endif
+}
+
+static inline gint
+xfce_notify_daemon_get_n_monitors (GdkScreen *screen)
+{
+#if GTK_CHECK_VERSION (3, 22, 0)
+    return gdk_display_get_n_monitors (gdk_screen_get_display (screen));
+#else
+    return gdk_screen_get_n_monitors (screen);
+#endif
+}
+
 static GdkFilterReturn
 xfce_notify_rootwin_watch_workarea(GdkXEvent *gxevent,
                                    GdkEvent *event,
@@ -205,15 +258,14 @@ xfce_notify_rootwin_watch_workarea(GdkXEvent *gxevent,
        && xndaemon->monitors_workarea)
     {
         GdkScreen *screen = gdk_event_get_screen(event);
-        int screen_number = gdk_screen_get_number (screen);
-        int nmonitor = gdk_screen_get_n_monitors (screen);
+        int nmonitor = xfce_notify_daemon_get_n_monitors (screen);
         int j;
 
         DBG("got _NET_WORKAREA change on rootwin!");
 
         for(j = 0; j < nmonitor; j++)
             xfce_notify_daemon_get_workarea(screen, j,
-                                            &(xndaemon->monitors_workarea[screen_number][j]));
+                                            &(xndaemon->monitors_workarea[j]));
     }
 
     return GDK_FILTER_CONTINUE;
@@ -226,39 +278,36 @@ xfce_notify_daemon_screen_changed(GdkScreen *screen,
     XfceNotifyDaemon *xndaemon = XFCE_NOTIFY_DAEMON(user_data);
     gint j;
     gint new_nmonitor;
-    gint screen_number;
     gint old_nmonitor;
 
     if(!xndaemon->monitors_workarea || !xndaemon->reserved_rectangles)
         /* Placement data not initialized, don't update it */
         return;
 
-    new_nmonitor = gdk_screen_get_n_monitors(screen);
-    screen_number = gdk_screen_get_number(screen);
+    new_nmonitor = xfce_notify_daemon_get_n_monitors (screen);
     old_nmonitor = GPOINTER_TO_INT(g_object_get_qdata(G_OBJECT(screen), XND_N_MONITORS));
 
-    DBG("Got 'screen-changed' signal for screen %d", screen_number);
+    DBG("Got 'screen-changed' signal");
 
     /* Set the new number of monitors */
     g_object_set_qdata(G_OBJECT(screen), XND_N_MONITORS, GINT_TO_POINTER(new_nmonitor));
 
     /* Free the current reserved rectangles on screen */
     for(j = 0; j < old_nmonitor; j++)
-        g_list_free(xndaemon->reserved_rectangles[screen_number][j]);
+        g_list_free(xndaemon->reserved_rectangles[j]);
 
-    g_free(xndaemon->reserved_rectangles[screen_number]);
-    g_free(xndaemon->monitors_workarea[screen_number]);
+    g_free(xndaemon->reserved_rectangles);
+    g_free(xndaemon->monitors_workarea);
 
-    xndaemon->monitors_workarea[screen_number] = g_new0(GdkRectangle, new_nmonitor);
+    xndaemon->monitors_workarea = g_new0(GdkRectangle, new_nmonitor);
     for(j = 0; j < new_nmonitor; j++) {
-        DBG("Screen %d changed, updating workarea for monitor %d", screen_number, j);
+        DBG("Screen changed, updating workarea for monitor %d", j);
         xfce_notify_daemon_get_workarea(screen, j,
-                                        &(xndaemon->monitors_workarea[screen_number][j]));
+                                        &(xndaemon->monitors_workarea[j]));
     }
 
     /* Initialize a new reserved rectangles array for screen */
-    xndaemon->reserved_rectangles[screen_number] = g_new0(GList *, new_nmonitor);
-    xndaemon->changed_screen = screen_number;
+    xndaemon->reserved_rectangles = g_new0(GList *, new_nmonitor);
 
     /* Traverse the active notifications tree to fill the new reserved rectangles array for screen */
     g_tree_foreach(xndaemon->active_notifications,
@@ -269,35 +318,27 @@ xfce_notify_daemon_screen_changed(GdkScreen *screen,
 static void
 xfce_notify_daemon_init_placement_data(XfceNotifyDaemon *xndaemon)
 {
-    gint nscreen = 1;
-    gint i;
+    GdkScreen *screen = gdk_screen_get_default();
+    gint nmonitor = xfce_notify_daemon_get_n_monitors (screen);
+    GdkWindow *groot;
+    int j;
 
-    xndaemon->reserved_rectangles = g_new(GList **, nscreen);
-    xndaemon->monitors_workarea = g_new(GdkRectangle *, nscreen);
+    g_object_set_qdata(G_OBJECT(screen), XND_N_MONITORS, GINT_TO_POINTER(nmonitor));
 
-    for(i = 0; i < nscreen; ++i) {
-        GdkScreen *screen = gdk_screen_get_default();
-        gint nmonitor = gdk_screen_get_n_monitors(screen);
-        GdkWindow *groot;
-        int j;
+    g_signal_connect(G_OBJECT(screen), "monitors-changed",
+                     G_CALLBACK(xfce_notify_daemon_screen_changed), xndaemon);
 
-        g_object_set_qdata(G_OBJECT(screen), XND_N_MONITORS, GINT_TO_POINTER(nmonitor));
+    xndaemon->reserved_rectangles = g_new0(GList *, nmonitor);
+    xndaemon->monitors_workarea = g_new0(GdkRectangle, nmonitor);
 
-        g_signal_connect(G_OBJECT(screen), "monitors-changed",
-                         G_CALLBACK(xfce_notify_daemon_screen_changed), xndaemon);
+    for(j = 0; j < nmonitor; j++)
+        xfce_notify_daemon_get_workarea(screen, j,
+                                        &(xndaemon->monitors_workarea[j]));
 
-        xndaemon->reserved_rectangles[i] = g_new0(GList *, nmonitor);
-        xndaemon->monitors_workarea[i] = g_new0(GdkRectangle, nmonitor);
-
-        for(j = 0; j < nmonitor; j++)
-            xfce_notify_daemon_get_workarea(screen, j,
-                                            &(xndaemon->monitors_workarea[i][j]));
-
-        /* Monitor root window changes */
-        groot = gdk_screen_get_root_window(screen);
-        gdk_window_set_events(groot, gdk_window_get_events(groot) | GDK_PROPERTY_CHANGE_MASK);
-        gdk_window_add_filter(groot, xfce_notify_rootwin_watch_workarea, xndaemon);
-    }
+    /* Monitor root window changes */
+    groot = gdk_screen_get_root_window(screen);
+    gdk_window_set_events(groot, gdk_window_get_events(groot) | GDK_PROPERTY_CHANGE_MASK);
+    gdk_window_add_filter(groot, xfce_notify_rootwin_watch_workarea, xndaemon);
 }
 
 
@@ -393,10 +434,6 @@ xfce_notify_daemon_init(XfceNotifyDaemon *xndaemon)
 
     /* CSS Styling provider  */
     xndaemon->css_provider = gtk_css_provider_new ();
-
-    xndaemon->close_timeout =
-        g_timeout_add_seconds(600, (GSourceFunc) xfce_notify_daemon_close_timeout,
-                              xndaemon);
 }
 
 static void
@@ -423,24 +460,17 @@ xfce_notify_daemon_finalize(GObject *obj)
 
 
     if(xndaemon->reserved_rectangles && xndaemon->monitors_workarea) {
-      gint nscreen, i, j;
+      gint i, j;
 
-      nscreen = 1;
+      GdkScreen *screen = gdk_screen_get_default ();
+      GdkWindow *groot = gdk_screen_get_root_window(screen);
+      gint nmonitor = xfce_notify_daemon_get_n_monitors (screen);
 
-      for(i = 0; i < nscreen; ++i) {
-          GdkScreen *screen = gdk_screen_get_default ();
-          GdkWindow *groot = gdk_screen_get_root_window(screen);
-          gint nmonitor = gdk_screen_get_n_monitors(screen);
+      gdk_window_remove_filter(groot, xfce_notify_rootwin_watch_workarea, xndaemon);
 
-          gdk_window_remove_filter(groot, xfce_notify_rootwin_watch_workarea, xndaemon);
-
-          for(j = 0; j < nmonitor; j++) {
-              if (xndaemon->reserved_rectangles[i][j])
-                  g_list_free(xndaemon->reserved_rectangles[i][j]);
-          }
-
-          g_free(xndaemon->reserved_rectangles[i]);
-          g_free(xndaemon->monitors_workarea[i]);
+      for(j = 0; j < nmonitor; j++) {
+          if (xndaemon->reserved_rectangles[j])
+              g_list_free(xndaemon->reserved_rectangles[j]);
       }
 
       g_free(xndaemon->reserved_rectangles);
@@ -490,28 +520,14 @@ xfce_notify_daemon_window_closed(XfceNotifyWindow *window,
     XfceNotifyDaemon *xndaemon = user_data;
     gpointer id_p = g_object_get_data(G_OBJECT(window), "--notify-id");
     GList *list;
-    gint screen = xfce_notify_window_get_last_screen(window);
     gint monitor = xfce_notify_window_get_last_monitor(window);
 
     /* Remove the reserved rectangle from the list */
-    list = xndaemon->reserved_rectangles[screen][monitor];
+    list = xndaemon->reserved_rectangles[monitor];
     list = g_list_remove(list, xfce_notify_window_get_geometry(window));
-    xndaemon->reserved_rectangles[screen][monitor] = list;
+    xndaemon->reserved_rectangles[monitor] = list;
 
     g_tree_remove(xndaemon->active_notifications, id_p);
-    if (g_tree_nnodes(xndaemon->active_notifications) == 0) {
-        /* All notifications expired */
-        /* Set a timeout to close xfce4-notifyd if it is idle
-         * for 10 minutes */
-
-        if(xndaemon->close_timeout)
-            g_source_remove(xndaemon->close_timeout);
-
-        xndaemon->close_timeout =
-            g_timeout_add_seconds(600,
-                                  (GSourceFunc) xfce_notify_daemon_close_timeout,
-                                  xndaemon);
-    }
 
     xfce_notify_gbus_emit_notification_closed (XFCE_NOTIFY_GBUS(xndaemon),
                                                GPOINTER_TO_UINT(id_p),
@@ -599,16 +615,23 @@ xfce_notify_daemon_get_workarea(GdkScreen *screen,
 
     DBG("Computing the workarea.");
 
+    display = gdk_screen_get_display(screen);
+
     /* Defaults */
+#if GTK_CHECK_VERSION (3, 22, 0)
+    gdk_monitor_get_geometry (gdk_display_get_monitor (display, monitor_num), workarea);
+#else
     gdk_screen_get_monitor_geometry(screen, monitor_num, workarea);
+#endif
 
     monitor_xoff = workarea->x;
     monitor_yoff = workarea->y;
 
     /* Sync the display */
-    display = gdk_screen_get_display(screen);
     gdk_display_sync(display);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     gdk_window_process_all_updates();
+G_GNUC_END_IGNORE_DEPRECATIONS
 
     windows_list = gdk_screen_get_window_stack(screen);
 
@@ -679,7 +702,7 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
     GdkDisplay *display;
     GdkDeviceManager *device_manager;
 #endif
-    gint x, y, monitor, screen_n, max_width;
+    gint x, y, monitor, max_width;
     GdkRectangle *geom_tmp, geom, initial, widget_geom;
     GList *list;
     gboolean found = FALSE;
@@ -700,12 +723,11 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
          * can happen. */
         GList *old_list;
 
-        screen_n = xfce_notify_window_get_last_screen(window);
         monitor = xfce_notify_window_get_last_monitor(window);
-        old_list = xndaemon->reserved_rectangles[screen_n][monitor];
+        old_list = xndaemon->reserved_rectangles[monitor];
 
         old_list = g_list_remove(old_list, xfce_notify_window_get_geometry(window));
-        xndaemon->reserved_rectangles[screen_n][monitor] = old_list;
+        xndaemon->reserved_rectangles[monitor] = old_list;
     }
 
     widget_screen = gtk_widget_get_screen (widget);
@@ -720,15 +742,14 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
 
     gdk_device_get_position (pointer, &p_screen, &x, &y);
 
-    if (xndaemon->primary_monitor == TRUE)
-        monitor = gdk_screen_get_primary_monitor(widget_screen);
+    if (xndaemon->primary_monitor == 1)
+        monitor = xfce_notify_daemon_get_primary_monitor (widget_screen);
     else
-        monitor = gdk_screen_get_monitor_at_point(p_screen, x, y);
-    screen_n = gdk_screen_get_number (p_screen);
+        monitor = xfce_notify_daemon_get_monitor_at_point (p_screen, x, y);
 
-    DBG("We are on the monitor %i, screen %i", monitor, screen_n);
+    DBG("We are on the monitor %i", monitor);
 
-    geom = xndaemon->monitors_workarea[screen_n][monitor];
+    geom = xndaemon->monitors_workarea[monitor];
 
     DBG("Workarea: (%i,%i), width: %i, height:%i",
         geom.x, geom.y, geom.width, geom.height);
@@ -768,7 +789,7 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
     max_width = 0;
 
     /* Get the list of reserved places */
-    list = xndaemon->reserved_rectangles[screen_n][monitor];
+    list = xndaemon->reserved_rectangles[monitor];
 
     if(!list) {
         /* If the list is empty, there are no displayed notifications */
@@ -776,10 +797,9 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
 
         xfce_notify_window_set_geometry(XFCE_NOTIFY_WINDOW(widget), widget_geom);
         xfce_notify_window_set_last_monitor(XFCE_NOTIFY_WINDOW(widget), monitor);
-        xfce_notify_window_set_last_screen(XFCE_NOTIFY_WINDOW(widget), screen_n);
 
         list = g_list_prepend(list, xfce_notify_window_get_geometry(XFCE_NOTIFY_WINDOW(widget)));
-        xndaemon->reserved_rectangles[screen_n][monitor] = list;
+        xndaemon->reserved_rectangles[monitor] = list;
 
         DBG("Notification position: x=%i y=%i", widget_geom.x, widget_geom.y);
         gtk_window_move(GTK_WINDOW(widget), widget_geom.x, widget_geom.y);
@@ -902,10 +922,9 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
 
     xfce_notify_window_set_geometry(XFCE_NOTIFY_WINDOW(widget), widget_geom);
     xfce_notify_window_set_last_monitor(XFCE_NOTIFY_WINDOW(widget), monitor);
-    xfce_notify_window_set_last_screen(XFCE_NOTIFY_WINDOW(widget), screen_n);
 
     list = g_list_prepend(list, xfce_notify_window_get_geometry(XFCE_NOTIFY_WINDOW(widget)));
-    xndaemon->reserved_rectangles[screen_n][monitor] = list;
+    xndaemon->reserved_rectangles[monitor] = list;
 
     DBG("Move the notification to: x=%i, y=%i", widget_geom.x, widget_geom.y);
     gtk_window_move(GTK_WINDOW(widget), widget_geom.x, widget_geom.y);
@@ -922,9 +941,6 @@ xfce_notify_daemon_update_reserved_rectangles(gpointer key,
     gint width, height;
     GtkAllocation allocation;
 
-    if(xfce_notify_window_get_last_screen(window) != xndaemon->changed_screen)
-      return;
-
     /* Get the size of the notification */
     gtk_window_get_size(GTK_WINDOW(window), &width, &height);
 
@@ -950,17 +966,6 @@ static gboolean notify_get_capabilities (XfceNotifyGBus *skeleton,
 
     xfce_notify_gbus_complete_get_capabilities(skeleton, invocation, capabilities);
 
-    if (g_tree_nnodes(xndaemon->active_notifications) == 0) {
-        /* No active notifications, reset the close timeout */
-        if(xndaemon->close_timeout)
-            g_source_remove(xndaemon->close_timeout);
-
-        xndaemon->close_timeout =
-            g_timeout_add_seconds(600,
-                                  (GSourceFunc) xfce_notify_daemon_close_timeout,
-                                  xndaemon);
-    }
-
     return TRUE;
 }
 
@@ -1089,6 +1094,7 @@ notify_update_known_applications (XfconfChannel *channel, gchar *new_app_name)
         }
     }
     xfconf_array_free (known_applications);
+    g_free(val);
 }
 
 static gboolean
@@ -1131,20 +1137,22 @@ notify_notify (XfceNotifyGBus *skeleton,
     GdkPixbuf *pix = NULL;
     GVariant *image_data = NULL;
     const gchar *image_path = NULL;
-    const gchar *desktop_id = NULL;
+    gchar *desktop_id = NULL;
     gchar *new_app_name;
     gint value_hint = 0;
     gboolean value_hint_set = FALSE;
     gboolean x_canonical = FALSE;
+    gboolean transient = FALSE;
     GVariant *item;
     GVariantIter iter;
     guint OUT_id;
+    gboolean application_is_muted = FALSE;
 
     g_variant_iter_init (&iter, hints);
 
     while ((item = g_variant_iter_next_value (&iter)))
     {
-        const char *key;
+        gchar *key;
         GVariant   *value;
 
         g_variant_get (item,
@@ -1160,24 +1168,31 @@ notify_notify (XfceNotifyGBus *skeleton,
                 /* don't expire urgent notifications */
                 expire_timeout = 0;
             }
+            g_variant_unref(value);
         }
         else if ((g_strcmp0 (key, "image_data") == 0) ||
                  (g_strcmp0 (key, "icon_data") == 0)  ||
                  (g_strcmp0 (key, "image-data") == 0) ||
                  (g_strcmp0 (key, "icon-data") == 0))
         {
+            if (image_data) {
+                g_variant_unref(image_data);
+            }
             image_data = value;
         }
         else if ((g_strcmp0 (key, "image-path") == 0) ||
                  (g_strcmp0 (key, "image_path") == 0))
         {
             image_path = g_variant_get_string (value, NULL);
+            g_variant_unref(value);
         }
         else if ((g_strcmp0 (key, "desktop_entry") == 0) ||
                  (g_strcmp0 (key, "desktop-entry") == 0))
         {
             if (g_variant_is_of_type (value, G_VARIANT_TYPE_STRING))
-                desktop_id = g_variant_get_string (value, NULL);
+                desktop_id = g_variant_dup_string (value, NULL);
+
+            g_variant_unref(value);
         }
         else if (g_strcmp0 (key, "value") == 0)
         {
@@ -1186,10 +1201,29 @@ notify_notify (XfceNotifyGBus *skeleton,
                 value_hint = g_variant_get_int32 (value);
                 value_hint_set = TRUE;
             }
+            g_variant_unref(value);
+        }
+        else if (g_strcmp0 (key, "transient") == 0)
+        {
+            transient = TRUE;
+            g_variant_unref(value);
         }
         else if (g_strcmp0 (key, "x-canonical-private-icon-only") == 0)
+        {
             x_canonical = TRUE;
+            g_variant_unref(value);
+        }
+        else if (g_strcmp0 (key, "urgency") == 0)
+        {
+            g_warning ("the urgency bit is set");
+            g_variant_unref(value);
+        }
+        else
+        {
+            g_variant_unref(value);
+        }
 
+        g_free(key);
         g_variant_unref (item);
     }
 
@@ -1203,22 +1237,35 @@ notify_notify (XfceNotifyGBus *skeleton,
     if(expire_timeout == -1)
         expire_timeout = xndaemon->expire_timeout;
 
+    application_is_muted = notify_application_is_muted (xndaemon->settings, new_app_name);
     /* Don't show notification bubbles in the "Do not disturb" mode or if the
        application has been muted by the user. Exceptions are "urgent"
        notifications which do not expire. */
     if (expire_timeout != 0)
     {
         if (xndaemon->do_not_disturb == TRUE ||
-            notify_application_is_muted (xndaemon->settings, new_app_name) == TRUE)
+            application_is_muted == TRUE)
         {
-            /* Reset the close timeout since we're processing a new notification,
-               even if we're not showing a notification bubble */
-            if(xndaemon->close_timeout)
-                g_source_remove(xndaemon->close_timeout);
+            /* Notifications marked as transient will never be logged */
+            if (xndaemon->notification_log == TRUE &&
+                transient == FALSE) {
+                /* Either log in DND mode or always for muted apps */
+                if (xndaemon->log_level == 0 && xndaemon->do_not_disturb == TRUE ||
+                    xndaemon->log_level == 1)
+                      /* Log either all, all except muted or only muted applications */
+                      if (xndaemon->log_level_apps == 0 ||
+                          xndaemon->log_level_apps == 1 && application_is_muted == FALSE ||
+                          xndaemon->log_level_apps == 2 && application_is_muted == TRUE)
+                          xfce_notify_log_insert (new_app_name, summary, body,
+                                                  image_data, image_path, app_icon,
+                                                  desktop_id, expire_timeout, actions);
+            }
 
-            xndaemon->close_timeout = 0;
-
-            xfce_notify_gbus_complete_notify(skeleton, invocation, OUT_id);
+            xfce_notify_gbus_complete_notify (skeleton, invocation, OUT_id);
+            if (image_data)
+                g_variant_unref (image_data);
+            if (desktop_id)
+                g_free (desktop_id);
             return TRUE;
         }
     }
@@ -1272,48 +1319,29 @@ notify_notify (XfceNotifyGBus *skeleton,
             xfce_notify_window_set_icon_pixbuf(window, pix);
             g_object_unref(G_OBJECT(pix));
         }
-        g_variant_unref(image_data);
     }
     else if (image_path) {
         xfce_notify_window_set_icon_name (window, image_path);
     }
     else if (app_icon && (g_strcmp0 (app_icon, "") != 0)) {
-        xfce_notify_window_set_icon_name(window, app_icon);
+        xfce_notify_window_set_icon_name (window, app_icon);
+    }
+    else if (desktop_id) {
+        xfce_notify_window_set_icon_name (window, notify_icon_name_from_desktop_id (desktop_id));
     }
-    else {
-        if(desktop_id) {
-            gchar *resource = g_strdup_printf("applications%c%s.desktop",
-                                              G_DIR_SEPARATOR,
-                                              desktop_id);
-            XfceRc *rcfile = xfce_rc_config_open(XFCE_RESOURCE_DATA,
-                                                 resource, TRUE);
-            if(rcfile) {
-                if(xfce_rc_has_group(rcfile, "Desktop Entry")) {
-                    const gchar *icon_file;
-                    xfce_rc_set_group(rcfile, "Desktop Entry");
-                    icon_file = xfce_rc_read_entry(rcfile, "Icon", NULL);
-                    if(icon_file) {
-                        pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                                       icon_file,
-                                                       48,
-                                                       GTK_ICON_LOOKUP_FORCE_SIZE,
-                                                       NULL);
 
-                        if(pix) {
-                            xfce_notify_window_set_icon_pixbuf(window, pix);
-                            g_object_unref(G_OBJECT(pix));
-                        }
-                    }
-                }
-                xfce_rc_close(rcfile);
-            }
-            g_free(resource);
-        }
-    }
+    if (xndaemon->notification_log == TRUE &&
+        xndaemon->log_level == 1 &&
+        xndaemon->log_level_apps <= 1 &&
+        transient == FALSE)
+        xfce_notify_log_insert (new_app_name, summary, body,
+                                image_data, image_path, app_icon,
+                                desktop_id, expire_timeout, actions);
 
     xfce_notify_window_set_icon_only(window, x_canonical);
 
-    xfce_notify_window_set_do_fadeout(window, xndaemon->do_fadeout);
+    xfce_notify_window_set_do_fadeout(window, xndaemon->do_fadeout, xndaemon->do_slideout);
+    xfce_notify_window_set_notify_location(window, xndaemon->notify_location);
 
     if(value_hint_set)
         xfce_notify_window_set_gauge_value(window, value_hint, xndaemon->css_provider);
@@ -1322,13 +1350,14 @@ notify_notify (XfceNotifyGBus *skeleton,
 
     gtk_widget_realize(GTK_WIDGET(window));
 
-    /* Remove close timeout as we display a new notification */
-    if(xndaemon->close_timeout)
-        g_source_remove(xndaemon->close_timeout);
-
-    xndaemon->close_timeout = 0;
     xfce_notify_gbus_complete_notify(skeleton, invocation, OUT_id);
 
+    g_free(new_app_name);
+    if (image_data)
+        g_variant_unref(image_data);
+    if (desktop_id)
+        g_free(desktop_id);
+
     return TRUE;
 }
 
@@ -1360,17 +1389,6 @@ static gboolean notify_get_server_information (XfceNotifyGBus *skeleton,
                                                      VERSION,
                                                      NOTIFICATIONS_SPEC_VERSION);
 
-    if (g_tree_nnodes(xndaemon->active_notifications) == 0) {
-        /* No active notifications, reset the close timeout */
-        if(xndaemon->close_timeout)
-            g_source_remove(xndaemon->close_timeout);
-
-        xndaemon->close_timeout =
-            g_timeout_add_seconds(600,
-                                  (GSourceFunc) xfce_notify_daemon_close_timeout,
-                                  xndaemon);
-    }
-
     return TRUE;
 }
 
@@ -1393,60 +1411,6 @@ static gboolean notify_quit (XfceNotifyOrgXfceNotifyd *skeleton,
 }
 
 
-
-static gboolean
-xfce_notify_daemon_close_timeout(gpointer data)
-{
-    daemon_quit (XFCE_NOTIFY_DAEMON(data));
-    return FALSE;
-}
-
-
-static GdkPixbuf *
-notify_pixbuf_from_image_data(GVariant *image_data)
-{
-    GdkPixbuf *pix = NULL;
-    gint32 width, height, rowstride, bits_per_sample, channels;
-    gboolean has_alpha;
-    GVariant *pixel_data;
-    gsize correct_len;
-    guchar *data;
-
-    if (!g_variant_is_of_type (image_data, G_VARIANT_TYPE ("(iiibiiay)")))
-    {
-        g_warning ("Image data is not the correct type");
-        return NULL;
-    }
-
-    g_variant_get (image_data,
-                   "(iiibii@ay)",
-                   &width,
-                   &height,
-                   &rowstride,
-                   &has_alpha,
-                   &bits_per_sample,
-                   &channels,
-                   &pixel_data);
-
-    correct_len = (height - 1) * rowstride + width
-                  * ((channels * bits_per_sample + 7) / 8);
-    if(correct_len != g_variant_get_size (pixel_data)) {
-        g_message ("Pixel data length (%lu) did not match expected value (%u)",
-                   g_variant_get_size (pixel_data), (guint)correct_len);
-        return NULL;
-    }
-
-    data = (guchar *) g_memdup (g_variant_get_data (pixel_data),
-                                g_variant_get_size (pixel_data));
-
-    pix = gdk_pixbuf_new_from_data(data,
-                                   GDK_COLORSPACE_RGB, has_alpha,
-                                   bits_per_sample, width, height,
-                                   rowstride,
-                                   (GdkPixbufDestroyNotify)g_free, NULL);
-    return pix;
-}
-
 static void
 xfce_notify_daemon_set_theme(XfceNotifyDaemon *xndaemon,
                              const gchar *theme)
@@ -1467,6 +1431,7 @@ xfce_notify_daemon_set_theme(XfceNotifyDaemon *xndaemon,
         g_free (file);
         file = g_strconcat("themes/", theme, "/xfce-notify-4.0/gtk.css", NULL);
         files = xfce_resource_lookup_all(XFCE_RESOURCE_DATA, file);
+        g_free(file);
         if (!files || !files[0])
         {
             g_warning ("theme '%s' is not found anywhere is user themes directories", theme);
@@ -1524,14 +1489,30 @@ xfce_notify_daemon_settings_changed(XfconfChannel *channel,
         xndaemon->do_fadeout = G_VALUE_TYPE(value)
                                ? g_value_get_boolean(value)
                                : TRUE;
+    } else if(!strcmp(property, "/do-slideout")) {
+       xndaemon->do_slideout = G_VALUE_TYPE(value)
+                               ? g_value_get_boolean(value)
+                               : FALSE;
     } else if(!strcmp(property, "/primary-monitor")) {
         xndaemon->primary_monitor = G_VALUE_TYPE(value)
-                                    ? g_value_get_boolean(value)
-                                    : TRUE;
+                                    ? g_value_get_uint(value)
+                                    : 0;
     } else if(!strcmp(property, "/do-not-disturb")) {
         xndaemon->do_not_disturb = G_VALUE_TYPE(value)
                                  ? g_value_get_boolean(value)
                                  : FALSE;
+    } else if(!strcmp(property, "/notification-log")) {
+        xndaemon->notification_log = G_VALUE_TYPE(value)
+                                     ? g_value_get_boolean(value)
+                                     : FALSE;
+    } else if(!strcmp(property, "/log-level")) {
+        xndaemon->log_level = G_VALUE_TYPE(value)
+                                  ? g_value_get_uint(value)
+                                  : 0;
+    } else if(!strcmp(property, "/log-level-apps")) {
+        xndaemon->log_level_apps = G_VALUE_TYPE(value)
+                                  ? g_value_get_uint(value)
+                                  : 0;
     }
 }
 
@@ -1565,12 +1546,26 @@ xfce_notify_daemon_load_config (XfceNotifyDaemon *xndaemon,
 
     xndaemon->do_fadeout = xfconf_channel_get_bool(xndaemon->settings,
                                                 "/do-fadeout", TRUE);
-    xndaemon->primary_monitor = xfconf_channel_get_bool(xndaemon->settings,
-                                                        "/primary-monitor", FALSE);
+
+    xndaemon->do_slideout = xfconf_channel_get_bool(xndaemon->settings,
+                                                "/do-slideout", FALSE);
+
+    xndaemon->primary_monitor = xfconf_channel_get_uint(xndaemon->settings,
+                                                        "/primary-monitor", 0);
 
     xndaemon->do_not_disturb = xfconf_channel_get_bool(xndaemon->settings,
                                                        "/do-not-disturb",
                                                        FALSE);
+
+    xndaemon->notification_log = xfconf_channel_get_bool(xndaemon->settings,
+                                                         "/notification-log",
+                                                         FALSE);
+    xndaemon->log_level = xfconf_channel_get_uint(xndaemon->settings,
+                                                        "/log-level",
+                                                        0);
+    xndaemon->log_level_apps = xfconf_channel_get_uint(xndaemon->settings,
+                                                        "/log-level-apps",
+                                                        0);
     /* Clean up old notifications from the backlog */
     xfconf_channel_reset_property (xndaemon->settings, "/backlog", TRUE);
 
