@@ -37,12 +37,11 @@
 #define DEFAULT_EXPIRE_TIMEOUT 10000
 #define DEFAULT_NORMAL_OPACITY 0.85
 #define DEFAULT_DO_FADEOUT     TRUE
+#define DEFAULT_DO_SLIDEOUT    FALSE
 #define FADE_TIME              800
 #define FADE_CHANGE_TIMEOUT    50
 #define DEFAULT_RADIUS         10
-#define DEFAULT_BORDER_WIDTH   2
 #define DEFAULT_PADDING        14.0
-#define BORDER                 6
 
 struct _XfceNotifyWindow
 {
@@ -50,13 +49,13 @@ struct _XfceNotifyWindow
 
     GdkRectangle geometry;
     gint last_monitor;
-    gint last_screen;
 
     guint expire_timeout;
 
     gboolean mouse_hover;
 
     gdouble normal_opacity;
+    gint original_x, original_y;
 
     guint32 icon_only:1,
             has_summary_text:1,
@@ -77,6 +76,8 @@ struct _XfceNotifyWindow
     guint op_change_steps;
     gdouble op_change_delta;
     gboolean do_fadeout;
+    gboolean do_slideout;
+    GtkCornerType notify_location;
 };
 
 typedef struct
@@ -175,10 +176,15 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     GtkWidget *tophbox, *vbox;
     gint screen_width;
     gdouble padding = DEFAULT_PADDING;
+#if GTK_CHECK_VERSION (3, 22, 0)
+    GdkMonitor *monitor;
+    GdkRectangle geometry;
+#endif
 
     window->expire_timeout = DEFAULT_EXPIRE_TIMEOUT;
     window->normal_opacity = DEFAULT_NORMAL_OPACITY;
     window->do_fadeout = DEFAULT_DO_FADEOUT;
+    window->do_slideout = DEFAULT_DO_SLIDEOUT;
 
     gtk_widget_set_name (GTK_WIDGET(window), "XfceNotifyWindow");
     gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
@@ -209,13 +215,20 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     /* Use the screen width to get a maximum width for the notification bubble.
        This assumes that a character is 10px wide and we want a third of the
        screen as maximum width. */
+#if GTK_CHECK_VERSION (3, 22, 0)
+    monitor = gdk_display_get_monitor_at_window (gtk_widget_get_display (GTK_WIDGET (window)),
+                                                 gtk_widget_get_window (GTK_WIDGET (window)));
+    gdk_monitor_get_geometry (monitor, &geometry);
+    screen_width = geometry.width / 30;
+#else
     screen_width = gdk_screen_get_width (screen) / 30;
+#endif
 
     gtk_widget_style_get(GTK_WIDGET(window),
                          "padding", &padding,
                          NULL);
 
-    tophbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, BORDER);
+    tophbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_box_set_homogeneous (GTK_BOX (tophbox), FALSE);
     gtk_container_set_border_width(GTK_CONTAINER(tophbox), padding);
     gtk_widget_show(tophbox);
@@ -223,13 +236,15 @@ xfce_notify_window_init(XfceNotifyWindow *window)
 
     window->icon_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_set_border_width(GTK_CONTAINER(window->icon_box), 0);
+    gtk_widget_set_margin_end (GTK_WIDGET (window->icon_box), padding);
+
     gtk_box_pack_start(GTK_BOX(tophbox), window->icon_box, FALSE, TRUE, 0);
 
     window->icon = gtk_image_new();
     gtk_widget_show(window->icon);
     gtk_container_add(GTK_CONTAINER(window->icon_box), window->icon);
 
-    window->content_box = vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, BORDER);
+    window->content_box = vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, padding / 2);
     gtk_box_set_homogeneous(GTK_BOX (vbox), FALSE);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
     gtk_widget_show(vbox);
@@ -239,20 +254,29 @@ xfce_notify_window_init(XfceNotifyWindow *window)
     gtk_widget_set_name (window->summary, "summary");
     gtk_label_set_max_width_chars (GTK_LABEL(window->summary), screen_width);
     gtk_label_set_line_wrap(GTK_LABEL(window->summary), TRUE);
+#if GTK_CHECK_VERSION (3, 16, 0)
     gtk_label_set_xalign (GTK_LABEL(window->summary), 0);
+#else
+    gtk_widget_set_halign (window->summary, GTK_ALIGN_START);
+#endif
     gtk_box_pack_start(GTK_BOX(vbox), window->summary, FALSE, FALSE, 0);
 
     window->body = gtk_label_new(NULL);
     gtk_widget_set_name (window->body, "body");
     gtk_label_set_max_width_chars (GTK_LABEL(window->body), screen_width);
     gtk_label_set_line_wrap(GTK_LABEL(window->body), TRUE);
+#if GTK_CHECK_VERSION (3, 16, 0)
     gtk_label_set_xalign (GTK_LABEL(window->body), 0);
+#else
+    gtk_widget_set_halign (window->body, GTK_ALIGN_START);
+#endif
+    gtk_widget_set_valign (window->body, GTK_ALIGN_START);
     gtk_box_pack_start(GTK_BOX(vbox), window->body, TRUE, TRUE, 0);
 
     window->button_box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
     gtk_button_box_set_layout(GTK_BUTTON_BOX(window->button_box),
                               GTK_BUTTONBOX_END);
-    gtk_box_set_spacing(GTK_BOX(window->button_box), BORDER / 2);
+    gtk_box_set_spacing(GTK_BOX(window->button_box), padding / 2);
     gtk_box_set_homogeneous(GTK_BOX(window->button_box), FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), window->button_box, FALSE, FALSE, 0);
 }
@@ -282,7 +306,6 @@ xfce_notify_window_start_expiration(XfceNotifyWindow *window)
                                           xfce_notify_window_expire_timeout,
                                           window);
     }
-
     gtk_widget_set_opacity(GTK_WIDGET(window), window->normal_opacity);
 }
 
@@ -487,7 +510,7 @@ static gboolean xfce_notify_window_draw (GtkWidget *widget,
     cairo_restore (cr);
 
     region = gdk_cairo_region_create_from_surface (surface);
-    if(!gtk_widget_is_composited(widget))
+    if(!gdk_screen_is_composited(gtk_widget_get_screen(widget)))
         gtk_widget_shape_combine_region(widget, region);
 
     /* however, of course always set the input shape; it doesn't matter
@@ -517,6 +540,9 @@ xfce_notify_window_enter_leave(GtkWidget *widget,
             if(window->fade_id) {
                 g_source_remove(window->fade_id);
                 window->fade_id = 0;
+                /* reset the sliding-out window to its original position */
+                if (window->do_slideout)
+                    gtk_window_move (GTK_WINDOW (window), window->original_x, window->original_y);
             }
         }
         gtk_widget_set_opacity(GTK_WIDGET(widget), 1.0);
@@ -564,6 +590,7 @@ xfce_notify_window_expire_timeout(gpointer data)
 {
     XfceNotifyWindow *window = data;
     gboolean          fade_transparent;
+    gint              animation_timeout;
 
     g_return_val_if_fail(XFCE_IS_NOTIFY_WINDOW(data), FALSE);
 
@@ -573,7 +600,14 @@ xfce_notify_window_expire_timeout(gpointer data)
         gdk_screen_is_composited(gtk_window_get_screen(GTK_WINDOW(window)));
 
     if(fade_transparent && window->do_fadeout) {
-        window->fade_id = g_timeout_add(FADE_CHANGE_TIMEOUT,
+        /* remember the original position of the window before we slide it out */
+        if (window->do_slideout) {
+            gtk_window_get_position (GTK_WINDOW (window), &window->original_x, &window->original_y);
+            animation_timeout = FADE_CHANGE_TIMEOUT / 2;
+        }
+        else
+            animation_timeout = FADE_CHANGE_TIMEOUT;
+        window->fade_id = g_timeout_add(animation_timeout,
                                         xfce_notify_window_fade_timeout,
                                         window);
     } else {
@@ -590,9 +624,25 @@ xfce_notify_window_fade_timeout(gpointer data)
 {
     XfceNotifyWindow *window = data;
     gdouble op;
+    gint x, y;
 
     g_return_val_if_fail(XFCE_IS_NOTIFY_WINDOW(data), FALSE);
 
+    /* slide out animation */
+    if (window->do_slideout) {
+        gtk_window_get_position (GTK_WINDOW (window), &x, &y);
+        if (window->notify_location == GTK_CORNER_TOP_RIGHT ||
+            window->notify_location == GTK_CORNER_BOTTOM_RIGHT)
+            x = x + 10;
+        else if (window->notify_location == GTK_CORNER_TOP_LEFT ||
+            window->notify_location == GTK_CORNER_BOTTOM_LEFT)
+            x = x - 10;
+        else
+            g_warning("Invalid notify location: %d", window->notify_location);
+        gtk_window_move (GTK_WINDOW (window), x, y);
+    }
+
+    /* fade-out animation */
     op = gtk_widget_get_opacity(GTK_WIDGET(window));
     op -= window->op_change_delta;
     if(op < 0.0)
@@ -720,7 +770,15 @@ xfce_notify_window_set_body(XfceNotifyWindow *window,
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
 
     if(body && *body) {
-        gtk_label_set_markup(GTK_LABEL(window->body), body);
+        if (pango_parse_markup (body, -1, 0, NULL, NULL, NULL, NULL)) {
+            gtk_label_set_markup (GTK_LABEL (window->body), body);
+        } else {
+            gchar *tmp;
+
+            tmp = g_markup_escape_text (body, -1);
+            gtk_label_set_text (GTK_LABEL (window->body), body);
+            g_free (tmp);
+        }
         gtk_widget_show(window->body);
 
         window->has_body_text = TRUE;
@@ -760,19 +818,6 @@ xfce_notify_window_get_last_monitor(XfceNotifyWindow *window)
    return window->last_monitor;
 }
 
-void
-xfce_notify_window_set_last_screen(XfceNotifyWindow *window,
-                                   gint screen)
-{
-    window->last_screen = screen;
-}
-
-gint
-xfce_notify_window_get_last_screen(XfceNotifyWindow *window)
-{
-    return window->last_screen;
-}
-
 void
 xfce_notify_window_set_icon_name (XfceNotifyWindow *window,
                                   const gchar *icon_name)
@@ -1104,11 +1149,21 @@ xfce_notify_window_unset_gauge_value(XfceNotifyWindow *window)
 
 void
 xfce_notify_window_set_do_fadeout(XfceNotifyWindow *window,
-                               gboolean do_fadeout)
+                                  gboolean do_fadeout,
+                                  gboolean do_slideout)
 {
     g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
 
     window->do_fadeout = do_fadeout;
+    window->do_slideout = do_slideout;
+}
+
+void xfce_notify_window_set_notify_location(XfceNotifyWindow *window,
+                                            GtkCornerType notify_location)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+
+    window->notify_location = notify_location;
 }
 
 void
